@@ -1,3 +1,11 @@
+var __rewriteRelativeImportExtension = (this && this.__rewriteRelativeImportExtension) || function (path, preserveJsx) {
+    if (typeof path === "string" && /^\.\.?\//.test(path)) {
+        return path.replace(/\.(tsx)$|((?:\.d)?)((?:\.[^./]+?)?)\.([cm]?)ts$/i, function (m, tsx, d, ext, cm) {
+            return tsx ? preserveJsx ? ".jsx" : ".js" : d && (!ext || !cm) ? m : (d + ext + "." + cm.toLowerCase() + "js");
+        });
+    }
+    return path;
+};
 import express from 'express';
 import history from 'connect-history-api-fallback';
 import cors from 'cors';
@@ -299,7 +307,9 @@ async function startGateway(settings) {
                     app: pluginsRouter,
                     logger: loggers.module(pluginName),
                 };
-                const instance = createPlugin(require(plugin), pluginsContext, pluginName);
+                const constructor = (await import(__rewriteRelativeImportExtension(plugin)))
+                    .default;
+                const instance = createPlugin(constructor, pluginsContext, pluginName);
                 plugins.push(instance);
                 logger.info(`Successfully loaded plugin ${instance.name}`);
             }
@@ -876,51 +886,182 @@ app.post('/api/settings', apisLimiter, isAuthenticated, async function (req, res
             throw Error('Gateway is restarting, wait a moment before doing another request');
         }
         let settings = req.body;
-        let restartAll = false;
+        let shouldRestart = false;
         let shouldRestartGw = false;
         let shouldRestartZniffer = false;
+        let canUpdateZwaveOptions = false;
         const actualSettings = jsonStore.get(store.settings);
         // TODO: validate settings using calss-validator
         // when settings is null consider a force restart
         if (settings && Object.keys(settings).length > 0) {
-            shouldRestartGw = !utils.deepEqual({
-                zwave: actualSettings.zwave,
-                gateway: actualSettings.gateway,
-                mqtt: actualSettings.mqtt,
-            }, {
-                zwave: settings.zwave,
-                gateway: settings.gateway,
-                mqtt: settings.mqtt,
-            });
+            // Check if gateway settings changed
+            const gatewayChanged = !utils.deepEqual(actualSettings.gateway, settings.gateway);
+            const mqttChanged = !utils.deepEqual(actualSettings.mqtt, settings.mqtt);
+            if (gatewayChanged || mqttChanged) {
+                shouldRestartGw = true;
+                shouldRestart = true;
+            }
+            let changedZwaveKeys = [];
+            // Check if Z-Wave settings changed
+            if (!utils.deepEqual(actualSettings.zwave, settings.zwave)) {
+                // These are ZwaveClient configuration properties that map to
+                // driver.updateOptions() parameters. The commented names show
+                // the corresponding driver option keys:
+                // - 'scales' maps to 'preferences.scales'
+                // - 'logEnabled', 'logLevel', etc. map to 'logConfig' properties
+                // - 'disableOptimisticValueUpdate' maps directly
+                const editableZWaveSettings = [
+                    'disableOptimisticValueUpdate',
+                    // preferences
+                    'scales',
+                    // logConfig
+                    'logEnabled',
+                    'logLevel',
+                    'logToFile',
+                    'maxFiles',
+                    'nodeFilter',
+                ];
+                // Find which Z-Wave settings actually changed
+                // Only check keys that exist in actual settings to avoid detecting
+                // new default properties added by the UI as "changed"
+                const allKeys = new Set([
+                    ...Object.keys(actualSettings.zwave || {}),
+                    ...Object.keys(settings.zwave || {}),
+                ]);
+                changedZwaveKeys = Array.from(allKeys).filter((key) => {
+                    return !utils.deepEqual(actualSettings.zwave?.[key], settings.zwave?.[key]);
+                });
+                logger.log('debug', 'Z-Wave settings changed: %o', {
+                    changedKeys: changedZwaveKeys,
+                    hasDriver: !!gw?.zwave?.driver,
+                });
+                // Check if only editable options changed
+                const onlyEditableChanged = changedZwaveKeys.every((key) => editableZWaveSettings.includes(key));
+                logger.log('debug', 'Checking if can update without restart: %o', {
+                    onlyEditableChanged,
+                    changedKeysLength: changedZwaveKeys.length,
+                    hasDriver: !!gw?.zwave?.driver,
+                });
+                if (onlyEditableChanged &&
+                    changedZwaveKeys.length > 0 &&
+                    gw?.zwave?.driver) {
+                    // Can update options without restart
+                    canUpdateZwaveOptions = true;
+                    logger.info('Z-Wave settings can be updated without restart');
+                }
+                else {
+                    // Need full restart
+                    shouldRestartGw = true;
+                    shouldRestart = true;
+                    logger.info('Z-Wave settings require full restart', {
+                        reason: !onlyEditableChanged
+                            ? 'non-editable settings changed'
+                            : changedZwaveKeys.length === 0
+                                ? 'no keys changed'
+                                : 'driver not available',
+                    });
+                }
+            }
+            // Check if Zniffer settings changed
             shouldRestartZniffer = !utils.deepEqual(actualSettings.zniffer, settings.zniffer);
-            // nothing changed, consider it a forced restart
-            restartAll = !shouldRestartGw && !shouldRestartZniffer;
+            if (shouldRestartZniffer) {
+                shouldRestart = true;
+            }
+            // Save settings to file
             await jsonStore.put(store.settings, settings);
+            // Update driver options if only editable options changed
+            if (canUpdateZwaveOptions && gw?.zwave?.driver) {
+                try {
+                    // Build editable options object with only changed properties
+                    // Map our settings to PartialZWaveOptions format
+                    const editableOptions = {};
+                    // Check disableOptimisticValueUpdate
+                    if (changedZwaveKeys.includes('disableOptimisticValueUpdate') &&
+                        settings.zwave?.disableOptimisticValueUpdate !==
+                            undefined) {
+                        editableOptions.disableOptimisticValueUpdate =
+                            settings.zwave.disableOptimisticValueUpdate;
+                    }
+                    // Check scales (maps to preferences.scales)
+                    if (changedZwaveKeys.includes('scales') &&
+                        settings.zwave?.scales !== undefined) {
+                        const preferences = utils.buildPreferences(settings.zwave || {});
+                        if (preferences) {
+                            editableOptions.preferences = preferences;
+                        }
+                    }
+                    // Check logConfig properties
+                    const logConfigChanged = [
+                        'logEnabled',
+                        'logLevel',
+                        'logToFile',
+                        'maxFiles',
+                        'nodeFilter',
+                    ].filter((key) => {
+                        return (changedZwaveKeys.includes(key) &&
+                            settings.zwave?.[key] !== undefined);
+                    }).length > 0;
+                    if (logConfigChanged) {
+                        // Build logConfig object from our settings
+                        editableOptions.logConfig = utils.buildLogConfig(settings.zwave || {});
+                    }
+                    if (Object.keys(editableOptions).length > 0) {
+                        gw.zwave.driver.updateOptions(editableOptions);
+                        logger.info('Updated Z-Wave driver options without restart:', Object.keys(editableOptions).join(', '));
+                    }
+                }
+                catch (error) {
+                    logger.error('Error updating driver options', error);
+                    // If update fails, require restart
+                    shouldRestart = true;
+                    shouldRestartGw = true;
+                }
+            }
         }
         else {
-            restartAll = true;
+            // Force restart if no settings provided
+            shouldRestart = true;
             settings = actualSettings;
         }
-        if (restartAll || shouldRestartGw) {
-            restarting = true;
-            await gw.close();
-            await destroyPlugins();
-            // reload loggers settings
-            setupLogging(settings);
-            // restart clients and gateway
-            await startGateway(settings);
-            backupManager.init(gw.zwave);
-        }
-        if (restartAll || shouldRestartZniffer) {
-            if (zniffer) {
-                await zniffer.close();
-            }
-            startZniffer(settings.zniffer);
-        }
         res.json({
             success: true,
-            message: 'Configuration updated successfully',
+            message: shouldRestart
+                ? 'Configuration saved. Restart required to apply changes.'
+                : 'Configuration updated successfully',
             data: settings,
+            shouldRestart,
+        });
+    }
+    catch (error) {
+        restarting = false;
+        logger.error(error);
+        res.json({ success: false, message: error.message });
+    }
+});
+// restart gateway
+app.post('/api/restart', apisLimiter, isAuthenticated, async function (req, res) {
+    try {
+        if (restarting) {
+            throw Error('Gateway is already restarting, wait a moment before doing another request');
+        }
+        const settings = jsonStore.get(store.settings);
+        restarting = true;
+        // Close gateway and restart
+        await gw.close();
+        await destroyPlugins();
+        if (settings.gateway) {
+            setupLogging({ gateway: settings.gateway });
+        }
+        await startGateway(settings);
+        backupManager.init(gw.zwave);
+        // Restart Zniffer if enabled
+        if (zniffer) {
+            await zniffer.close();
+        }
+        startZniffer(settings.zniffer);
+        res.json({
+            success: true,
+            message: 'Gateway restarted successfully',
         });
     }
     catch (error) {
